#include <vector>
#include "defs.h"
#include "timer.h"
#include "alloc.h"


void hip_memcopy(const std::vector<int> &params, olocal &output) {
    int rank = params[0];
    int distance = params[1];
    int ngpus = params[2];
    int pinned = params[3];

    if (rank % distance) 
        return;

    real *d = nullptr, *h = nullptr;
    size_t N = 1<<26;
    int niters = 10, nwarmup = 3;
    if (pinned) {
        h = (real *)sys::cpu_alloc_pinned(N * sizeof(real));
    } else {
        h = (real *)sys::cpu_alloc_pageable(N * sizeof(real));
    }
    d = (real *)sys::gpu_alloc(N * sizeof(real));
    print(rank, std::string("GPU-CPU (Device-Host) memcopy benchmark") + (pinned?"pinned":"pageable") + ":");
    if (distance == 1) {
        print(rank, "Using each GPU:");
    } else {
        print(rank, std::string("Using every ") + std::to_string(distance) + " GPU (" + std::to_string(ngpus/distance) + " GPUs per node)");
    }

    std::vector<double> result;

    {
        timer t(rank2str(rank) + "H2D", sizeof(real)*N, niters, result);
        for (int i = 0; i < niters + nwarmup; i++) {
            if (i == nwarmup)
                t.start();
            HIP_CALL(hipMemcpy(h, d, sizeof(real)*N, hipMemcpyHostToDevice));
        }
    }
    {
        timer t(rank2str(rank) + "D2H", sizeof(real)*N, niters, result);
        for (int i = 0; i < niters + nwarmup; i++) {
            if (i == nwarmup)
                t.start();
            HIP_CALL(hipMemcpy(d, h, sizeof(real)*N, hipMemcpyDeviceToHost));
        }
    }
    sys::gpu_free(d);
    if (pinned) {
        sys::cpu_free_pinned(h);
    } else {
        sys::cpu_free_pageable(h);
    }
    output.values.push_back(result);
    output.header = "H2D_D2H_";
    output.header += (pinned?"pinned":"pageable");
}

